#include <stdio.h>
#include "matrix.h"

#define INFINITY (10000000l)

struct Edge
{ int v1,v2;  // vertices making up edge
  long cost; // cost of the edge
  Edge(int u1,int u2,long c): v1(u1),v2(u2),cost(c)
	 {}
};

class FloydWarshall
{  int n; // number vertices of graph
	Matrix<long> A; // matrix of minimum costs
	Matrix<long> cost; // cost adjacency matrix
	Matrix<int> parentTrees;
	 // the actual shortest paths; parentTree[i] is the tree of shortest paths
	 // with source vertex i

	public:
	  FloydWarshall(Edge* e,int cv,int ce); // e is array of edges of graph
										// cv number of vertices, ce number of edges

	  void allPairs();
	  // computes the costs A of all shortest paths
	  // and the actual shortest paths in parentTrees

	  void showResults();
};

FloydWarshall::FloydWarshall(Edge* e,int vertexCount,int edgeCount)
  :n(vertexCount),A(n+1,n+1),cost(n+1,n+1),parentTrees(n+1,n+1)
{ int i,j,u,v;
  for (i=1;i<=n;i++)
	 for (j=1;j<=n;j++)
		cost(i,j)=(i != j) ? INFINITY : 0;
  for (i=0;i<edgeCount;i++) {
	 u=e[i].v1; v=e[i].v2;
	 cost(u,v)=e[i].cost;
  }
}


void FloydWarshall::allPairs()
/*  This function computes the costs of the shortest paths from vertex i to
	 vertex j for all pairs i,j and the actual shortes paths from i to j.
	 The minimum costs are stored in A and the paths in parentTrees.  Here
	 costs can be negative, but the graph can not have any negative cost cycles.
*/
{  int i,j,k,u,v;
	long sum;
	for (i=1;i<=n;i++)
	  for (j=1;j<=n;j++) {
		 A(i,j)=cost(i,j);
		 if (i != j && cost(i,j) < INFINITY)
			parentTrees(i,j)=i;
		 else
			parentTrees(i,j)=0;
	  }
	for (k=1;k<=n;k++)
	  for (i=1;i<=n;i++)
		 for (j=1;j<=n;j++) {
			sum=A(i,k)+A(k,j);
			if (A(i,k)<INFINITY && A(k,j)<INFINITY && sum<A(i,j)) {
			  A(i,j)=sum;
			  // You update parentTrees appropriately
			}
		 }
}

void FloydWarshall::showResults()
// Just outputs the minimum costs matrix A and checks the shortest paths
// in parentTrees for agreement with entry in A
{ int i,j,u,v;
  printf("Here are the minimum costs for shortest paths\n");
  for (i=1;i<=n;i++) {
	 for (j=1;j<=n;j++)
		printf("%ld ",A(i,j));
	 printf("\n");
  }
  long sum;
  int *reversePath=new int[n];
  int m;
  printf("\nHere are the shortest paths:\n\n");
  for (i=1;i<=n;i++)
	 for (j=1;j<=n;j++) {
		sum=0;
		m=0;
		for (u=j,v=parentTrees(i,j);v;u=v,v=parentTrees(i,u)) {
		  sum+=cost(v,u);
		  reversePath[m++]=u;
		}
		reversePath[m]=u;
		if (A(i,j)<INFINITY)
		  if (sum !=A(i,j)) {
			 printf("Error at vertices %d %d\n",i,j);
			 printf("A[%d][%d] = %ld and path sum is %ld\n",i,j,A(i,j),sum);
		  }
		  else {
			 printf("%d to %d: ",i,j);
			 for (;m>=0;m--)
				printf("%d ",reversePath[m]);
			 printf("\n");
		  }
	 }
  delete [] reversePath;
}


Edge e[] = { /*Edge(1,2,55),
				 Edge(1,3,25),
				 Edge(1,4,45),
				 Edge(2,5,5),
				 Edge(3,5,40),
				 Edge(4,5,20),
				 Edge(4,8,30),
				 Edge(5,6,35),
				 Edge(5,7,15),
				 Edge(7,8,50),
				 Edge(6,7,10),
				 Edge(2,1,55),
				 Edge(3,1,25),
				 Edge(4,1,45),
				 Edge(5,2,5),
				 Edge(5,3,40),
				 Edge(5,4,20),
				 Edge(8,4,30),
				 Edge(6,5,35),
				 Edge(7,5,15),
				 Edge(8,7,50),
				 Edge(7,6,10)
			  };  */

				 Edge(1,2,4),
				 Edge(1,3,5),
				 Edge(3,2,-4),
				 Edge(2,4,6),
				 Edge(4,3,3),
				 Edge(4,5,2),
				 Edge(4,6,9),
				 Edge(5,7,1),
				 Edge(6,7,3)
			  };
main()
{ FloydWarshall G(e,7,9);
  G.allPairs();
  G.showResults();
  return(0);
}